@@ -2,29 +2,30 @@
 //   input.c
 //
 //   Project:  EPA SWMM5
-//   Version:  5.1
-//   Date:     03/20/14  (Build 5.1.001)
-//             09/15/14  (Build 5.1.007)
-//             08/01/16  (Build 5.1.011)
-//             04/01/20  (Build 5.1.015)
+//   Version:  5.2
+//   Date:     06/01/22  (Build 5.2.1)
 //   Author:   L. Rossman
 //
 //   Input data processing functions.
 //
+//   Update History
+//   ==============
 //   Build 5.1.007:
 //   - Support added for climate adjustment input data.
-//
 //   Build 5.1.011:
 //   - Support added for reading hydraulic event dates.
-//
 //   Build 5.1.015:
 //   - Support added for multiple infiltration methods within a project.
+//   Build 5.2.0:
+//   - Support added for Streets and Inlets.
+//   - Support added for named variables & math expressions in control rules.
+//   Build 5.2.1:
+//   - Possible integer underflow avoided in getTokens() function.
 //-----------------------------------------------------------------------------
 #define _CRT_SECURE_NO_DEPRECATE
 
 #include <stdlib.h>
 #include <string.h>
-#include <stdlib.h>
 #include <math.h>
 #include "headers.h"
 #include "lid.h"
@@ -87,14 +88,15 @@ int input_countObjects()
     for (i = 0; i < MAX_OBJ_TYPES; i++) Nobjects[i] = 0;
     for (i = 0; i < MAX_NODE_TYPES; i++) Nnodes[i] = 0;
     for (i = 0; i < MAX_LINK_TYPES; i++) Nlinks[i] = 0;
+    controls_init();
 
     // --- make pass through data file counting number of each object
     while ( fgets(line, MAXLINE, Finp.file) != NULL )
     {
         // --- skip blank lines & those beginning with a comment
         lineCount++;
-        strcpy(wLine, line);           // make working copy of line
-        tok = strtok(wLine, SEPSTR);   // get first text token on line
+        sstrncpy(wLine, line, MAXLINE);     // make working copy of line
+        tok = strtok(wLine, SEPSTR);        // get first text token on line
         if ( tok == NULL ) continue;
         if ( *tok == ';' ) continue;
 
@@ -176,7 +178,7 @@ int input_readData()
     {
         // --- make copy of line and scan for tokens
         lineCount++;
-        strcpy(wLine, line);
+        sstrncpy(wLine, line, MAXLINE);
         Ntokens = getTokens(wLine);
 
         // --- skip blank lines and comments
@@ -184,12 +186,12 @@ int input_readData()
         if ( *Tok[0] == ';' ) continue;
 
         // --- check if max. line length exceeded
-        lineLength = strlen(line);
+        lineLength = (int)strlen(line);
         if ( lineLength >= MAXLINE )
         {
             // --- don't count comment if present
             comment = strchr(line, ';');
-            if ( comment ) lineLength = comment - line;    // Pointer math here
+            if ( comment ) lineLength = (int)(comment - line);  // Pointer math here
             if ( lineLength >= MAXLINE )
             {
                 inperr = ERR_LINE_LENGTH;
@@ -407,6 +409,7 @@ int  addObject(int objType, char* id)
 
       case s_CONTROL:
         if ( match(id, w_RULE) ) Nobjects[CONTROL]++;
+        else controls_addToCount(id);
         break;
 
       case s_TRANSECT:
@@ -436,6 +439,22 @@ int  addObject(int objType, char* id)
         break;
 
       case s_EVENT: NumEvents++; break;
+
+      case s_STREET:
+        if ( !project_addObject(STREET, id, Nobjects[STREET]) )
+            errcode = error_setInpError(ERR_DUP_NAME, id);
+        Nobjects[STREET]++;
+        break;
+
+      case s_INLET:
+        // --- an INLET object can span several lines
+        if (project_findObject(INLET, id) < 0)
+        {
+            if ( !project_addObject(INLET, id, Nobjects[INLET]) ) 
+                errcode = error_setInpError(ERR_DUP_NAME, id);
+            Nobjects[INLET]++;
+        }
+        break;
     }
     return errcode;
 }
@@ -481,7 +500,7 @@ int  parseLine(int sect, char *line)
         return subcatch_readSubareaParams(Tok, Ntokens);
 
       case s_INFIL:
-        return infil_readParams(InfilModel, Tok, Ntokens);                     //(5.1.015)
+        return infil_readParams(InfilModel, Tok, Ntokens);
 
       case s_AQUIFER:
         j = Mobjects[AQUIFER];
@@ -600,6 +619,15 @@ int  parseLine(int sect, char *line)
       case s_EVENT:
         return readEvent(Tok, Ntokens);
 
+      case s_STREET:
+        return street_readParams(Tok, Ntokens);
+
+      case s_INLET:
+        return inlet_readDesignParams(Tok, Ntokens);
+
+      case s_INLET_USAGE:
+        return inlet_readUsageParams(Tok, Ntokens);
+
       default: return 0;
     }
 }
@@ -620,6 +648,11 @@ int readControl(char* tok[], int ntoks)
     // --- check for minimum number of tokens
     if ( ntoks < 2 ) return error_setInpError(ERR_ITEMS, "");
 
+    if (match(tok[0], w_VARIABLE))
+        return controls_addVariable(tok, ntoks);
+    if (match(tok[0], w_EXPRESSION))
+        return controls_addExpression(tok, ntoks);
+
     // --- get index of control rule keyword
     keyword = findmatch(tok[0], RuleKeyWords);
     if ( keyword < 0 ) return error_setInpError(ERR_KEYWORD, tok[0]);
@@ -672,7 +705,7 @@ int readTitle(char* line)
         if ( strlen(Title[i]) == 0 )
         {
             // --- strip line feed character from input line
-            n = strlen(line);
+            n = (int)strlen(line);
             if (line[n-1] == 10) line[n-1] = ' ';
 
             // --- copy input line into Title entry
@@ -771,19 +804,19 @@ int  match(char *str, char *substr)
 //           (not case sensitive).
 //
 {
-    int i,j;
+    int i,j,k;
 
     // --- fail if substring is empty
     if (!substr[0]) return(0);
 
     // --- skip leading blanks of str
-    for (i = 0; str[i]; i++)
+    for (k = 0; str[k]; k++)
     {
-        if (str[i] != ' ') break;
+        if (str[k] != ' ') break;
     }
 
     // --- check if substr matches remainder of str
-    for (i = i,j = 0; substr[j]; i++,j++)
+    for (i = k,j = 0; substr[j]; i++,j++)
     {
         if (!str[i] || UCHAR(str[i]) != UCHAR(substr[j])) return(0);
     }
@@ -858,7 +891,8 @@ int  getTokens(char *s)
 //           in CONSTS.H. Text between quotes is treated as a single token.
 //
 {
-    int  len, m, n;
+    int  len, n;
+    int  m;
     char *c;
 
     // --- begin with no tokens
@@ -868,12 +902,12 @@ int  getTokens(char *s)
     // --- truncate s at start of comment 
     c = strchr(s,';');
     if (c) *c = '\0';
-    len = strlen(s);
+    len = (int)strlen(s);
 
     // --- scan s for tokens until nothing left
     while (len > 0 && n < MAXTOKS)
     {
-        m = strcspn(s,SEPSTR);              // find token length 
+        m = (int)strcspn(s,SEPSTR);         // find token length 
         if (m == 0) s++;                    // no token found
         else
         {
@@ -881,7 +915,7 @@ int  getTokens(char *s)
             {
                 s++;                        // start token after quote
                 len--;                      // reduce length of s
-                m = strcspn(s,"\"\n");      // find end quote or new line
+                m = (int)strcspn(s,"\"\n"); // find end quote or new line
             }
             s[m] = '\0';                    // null-terminate the token
             Tok[n] = s;                     // save pointer to token 
@@ -890,7 +924,7 @@ int  getTokens(char *s)
         }
         len -= m+1;                         // update length of s
     }
-    return(n);
+    return n;
 }
 
 //=============================================================================
