@@ -1,5 +1,8 @@
 from html import entities
-from . import swmm5
+if __name__=='__main__':
+    import swmm5
+else:
+    from . import swmm5
 from collections import OrderedDict
 from tempfile import mkstemp
 from os.path import basename, dirname
@@ -16,7 +19,7 @@ class SWMM5Error(Exception):
 
     def __init__(self,value):
         self.value=value
-        self.msg=swmm5.error_getMsg(value, '')
+        self.msg=swmm5.error_getMsg(value, '') # hack need this empty string! (as of 2022-APR-17)
     def __str__(self):
         return repr(self.value)+": "+self.msg
 
@@ -204,8 +207,8 @@ class SWMM5Simulation(object):
             _results=OrderedDict()
             for withinentiry in self.entityList(within=topentity):
                 __results=OrderedDict()
-                for index, value in self._variables[topentity]:
-                    res=self.Results(topentity, withinentiry, index)
+                for index, value in enumerate(self._variables[topentity]):
+                    res=list(self.Results(topentity, withinentiry, index))
                     __results[value]=res
                 _results[withinentiry]=__results
             results[topentity]=_results
@@ -220,3 +223,5 @@ if __name__=="__main__":
     print(ss.SWMM_Nperiods)
     print (list(ss.Results('NODE','J1', 4)))
     print (ss.SWMM5_Version())
+    ar=ss.allResults()
+    print(ar)
